$MOD51
$Title(DT51 I2C ADDA DRIVER REV3 4/07/02)
;edited on 24/02/03 to change duplicate labels (with KND)

;SCL P1.6
;SDA P1.7

Flag		DATA	0021H
FACK		BIT	Flag.0
FSRDY		BIT	Flag.1
FLB		BIT	Flag.2
AutoInc		BIT	Flag.3
OutputEnb	BIT	Flag.4
InitDone	BIT	Flag.5
MCOk		BIT	Flag.6
First		BIT	Flag.7

Ch0		DATA	30H
Ch1		DATA	31H
Ch2		DATA	32H
Ch3		DATA	33H
Mode		DATA	34H		;0-3	
Channel		DATA	35H		;0-3
ADDACB		DATA	36H
NCh		DATA	37H
LCh		DATA	38H		

		CSEG
;-----------------------
;Check SCL Line Routine
;Input  : none
;Output : FSRDY=0/1
;-----------------------
ChkSCLCon:	JB	P1.6,SlvRdy		;Check SCL
		CLR	FSRDY
		RET
SlvRdy:		SETB	FSRDY
		RET
;-----------------------
;Start Condition Routine
;Input  : none
;Output : none
;----------------------- 
StartCon:	CLR	P1.6			;SCL Low
		SETB	P1.7			;SDA High
		SETB	P1.6			;SCL High	
		CLR	P1.7			;SDA ~|_ when SCL High 
		LCALL	Delay5us		;
		RET				
;---------------------
;Delay Routine For I2C
;Input  : none
;Output : none
;--------------------- 
Delay5us:	NOP
		RET
Delay3us:	RET
;-----------------------------------------
;Master Transmitter Routine
;Input  : ACC <- Data/Address/Control byte
;Output : none
;----------------------------------------- 
Mtx:		PUSH	07H
		PUSH	ACC
		MOV	R7,#8
NexBit:		CLR	P1.6			;SCL Low -------------\
		RLC	A 			;(1) Data MSB first   |
		JNC	TxL 			;(2) Data Bit Low     |	
TxH:		SETB	P1.7 			;(2) SDA High	      |-1 CLK
		AJMP	Clock 			;(2)  		      |
TxL:		CLR	P1.7 			;(2) SDA Low	      |
Clock:		SETB	P1.6			;SCL High             |
		ACALL	Delay3us		; 		      |
		DJNZ	R7,NexBit		;(2) -----------------/ 				
AckPhaseClk:	CLR	P1.6			;SCL Low -------------\
		SETB	P1.7			;(2) release SDA high  |
		ACALL	Delay3us		;		       |
		SETB	P1.6			;SCL High              |-CLK
		JNB	P1.7,Ack		;(3) Chck SDA ACK/NACK | ACK
Nack:		CLR	FACK			;(2)		       |
		AJMP	EOMtx			;(2)   		       |	
Ack:		SETB	FACK			;(2)		       |	 
EOMTx:		POP	ACC			;(2)		       |	
		POP	07H			;(2)		       |		       		
		RET				;(1) ------------------/
;-----------------------
;Stop Condition Routine
;Input  : none
;Output : none
;----------------------- 
StopCon:	CLR	P1.6			;SCL Low------------------\
		CLR	P1.7 			;(2) SDA Low		  |
		ACALL	Delay3us		;			  |
		SETB	P1.6			;SCL High		  |-STOP
		SETB	P1.7			;(2) SDA _|~ when SCL High|
		NOP				;(1)			  |
		NOP				;(1)			  |
		RET				;(1) ---------------------/
;-----------------------
;Master Receiver Routine
;Input  : FLB <- 1/0	
;Output : ACC
;----------------------- 
MRx:		PUSH	07H
RdData:		MOV	R7,#8
		CLR	C
NextBit:	CLR	P1.6			;SCL Low --------\	
		SETB	P1.7			;(2)For input	 |
		ACALL	Delay3us		;		 |
		SETB	P1.6			;SCL High	 |
		JB	P1.7,HiBit		;(3) Chk SDA H/L |
LoBit:		CLR	C			;(1)		 |-1 CLK
		AJMP	SaveBit			;(2)		 |
HiBit:		SETB	C			;(1)		 |
SaveBit:	RLC	A			;(1)		 |
		DJNZ	R7,NextBit		;(2) ------------/
AckPhsClk:	CLR	P1.6			;SCL Low ----\	
		JB	FLB,NAckd		;(3)	     |	
Ackd:		CLR	P1.7			;(2)	     |		
		AJMP	ClkHigh			;(2)  	     |-ACK Related
NAckd:		SETB	P1.7			;(2)	     | CLK
ClkHigh:	SETB	P1.6			;SCL High    |	
		ACALL	Delay3us		; -----------/	
EOMRx:		POP	07H			;(2)
		RET				;(1)

;----------------------------
;Check Valid Channel and Mode
;Input  : Mode,Channel
;Output : Channel, MCOk Flag
;----------------------------
ChkMC:		PUSH	ACC
		MOV	A,Mode
CM0:		CJNE	A,#0,CM1
Mode0:		MOV	NCh,#3
		MOV	LCh,#Mode
		MOV	A,Channel
		CJNE	A,#4,$+3
		JC	ValidMC
		MOV	Channel,#3
		LJMP	ValidMC
CM1:		CJNE	A,#1,CM2
Mode1:		MOV	NCh,#2
		MOV	LCh,#Ch3
		MOV	A,Channel
		CJNE	A,#3,$+3
		JC	ValidMC
		MOV	Channel,#2
		LJMP	ValidMC				
CM2:		CJNE	A,#2,CM3
Mode2:		MOV	NCh,#2
		MOV	LCh,#Ch3
		MOV	A,Channel
		CJNE	A,#3,$+3
		JC	ValidMC
		MOV	Channel,#2
		LJMP	ValidMC				
CM3:		CJNE	A,#3,InvalidMC
Mode3:		MOV	NCh,#1
		MOV	LCh,#Ch2
		MOV	A,Channel		
		CJNE	A,#2,$+3
		JC	ValidMC
		MOV	Channel,#1
		LJMP	ValidMC				
InvalidMC:	CLR	MCOk
		POP	ACC
		RET
ValidMC:	SETB	MCOk
		POP	ACC
		RET

;---------------------------------------
;Init ADDA Mode Routine
;Input  : Mode,Channel,AutoInc,OutputEnb
;  	  A <- Slave Address (0-7)
;Output : ADDACB,InitDone Flag
;---------------------------------------
InitADDA:  	PUSH	ACC
		PUSH	B
		LCALL	ChkMC
		JNB	MCOk,InitADDAbort
		XCH	A,B
		MOV	A,Mode
		SWAP	A
		ORL	A,Channel
ChkAutoInc:	JNB	AutoInc,ChkOutputEnb
   		SETB	ACC.6
		SETB	ACC.2
ChkOutputEnb:	JNB	OutputEnb,NextInit
		SETB	ACC.6
NextInit:	XCH	A,B
		RL	A
		ANL	A,#0EH			;0000XXX0B
		ORL	A,#90H			;1001XXX0B
		LCALL	StartCon
		LCALL	MTx
		JNB	FACK,InitADDAbort
		XCH	A,B
		LCALL	MTx
		JNB	FACK,InitADDAbort
InitADDASucc:	LCALL	StopCon
		MOV	ADDACB,A
		SETB	InitDone
		POP	B
		POP	ACC
		RET	
InitADDAbort:	LCALL	StopCon
		CLR	InitDone
		POP	B
		POP	ACC
		RET	

;-------------------------------------
;Read ADC Routine
;Input  : A <- Slave Address (0-7) 
;Output : Ch0-Ch3
;-------------------------------------
ReadADC:	PUSH	ACC
		PUSH	00H
		PUSH	07H
		JNB	InitDone,ExitReadADC
		LCALL	ChkSCLCon
		JNB	FSRDY,ExitReadADC
		RL	A
		ANL	A,#0FH			;0000XXXXB
		ORL	A,#91H			;1001XXX1B
		JNB	AutoInc,ReadADCChx
ReadADCAI:	PUSH	ACC
		MOV	R0,#Ch0
		MOV	A,Channel
		ADD	A,R0
		MOV	R0,A
		POP	ACC
		LCALL	StartCon
		LCALL	MTx
		JNB	FACK,ReadADCAbort
		CLR	FLB
		LCALL	MRx
		MOV	R7,NCh
NxtCh:		LCALL	MRx
		MOV	@R0,A
		INC	R0
		MOV	A,R0
		CJNE	A,LCh,GoAhead		
		MOV	R0,#Ch0
GoAhead:	DJNZ	R7,NxtCh
		SETB	FLB
		LCALL	MRx
		MOV	@R0,A
		LCALL	StopCon
		AJMP	ExitReadADC
ReadADCChx:	MOV	R0,#Channel
		LCALL	StartCon
		LCALL	MTx
		JNB	FACK,ReadADCAbort
		CLR	FLB
		LCALL	MRx
		SETB	FLB		
		LCALL	MRx
		LCALL	StopCon
ChkCh0:		CJNE	@R0,#0,ChkCh1
		MOV	Ch0,A
		AJMP	ExitReadADC
ChkCh1:		CJNE	@R0,#1,ChkCh2
		MOV	Ch1,A
		AJMP	ExitReadADC
ChkCh2:		CJNE	@R0,#2,ADCCh3
		MOV	Ch2,A
		AJMP	ExitReadADC
ADCCh3:		MOV	Ch3,A
		AJMP	ExitReadADC
ReadADCAbort:	LCALL	StopCon
ExitReadADC:	POP	07H
		POP	00H
		POP	ACC
		RET

;---------------------------------
;Write to DAC Routine
;Input  : A <- Slave Address (0-7)
;         B <- DAC Data
;Output : Output Enable Flag Set
;---------------------------------
WriteDAC:	PUSH	ACC
		PUSH	B
		JNB	InitDone,ExitWriteDAC
		LCALL	ChkSCLCon
		JNB	FSRDY,ExitWriteDAC
		RL	A
		ANL	A,#0EH			;0000XXX0B
		ORL	A,#90H			;1001XXX0B				
		LCALL	StartCon
		LCALL	Mtx
		JNB	FACK,WriteDACAbort
		MOV	A,ADDACB
		SETB	ACC.6
		LCALL	Mtx
		JNB	FACK,WriteDACAbort
		XCH	A,B
		LCALL	MTx		
WriteDACAbort:	LCALL	StopCon
ExitWriteDAC:	POP	B
		POP	ACC
		RET
